"""
Provides modules for creation anonymous variables to help minimize the setup/arrange phase when writing unit tests
"""

__version__ = "0.1.0"

from autofaker.autodata import *
from autofaker.decorators import *
